/*
 *    snake, a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "modules.h"
#include <stdlib.h>		/* malloc */
#include <string.h>		/* strings */

/* local definitions */
typedef struct t_brain
{
  int kl;			/* left key */
  int ku;			/* up key */
  int kr;			/* right key */
  int kd;			/* down key */
  int ks;			/* stop key */
  int kg;			/* grow key */
}
brain;

/* private functions */
static void play (snake *);
static brain *make_brain (int kl, int ku, int kr, int kd, int ks, int kg);

/* public functions */
int human_module_init (void);


int
human_module_init ()
{
  snake *mysnake;
  brain *mybrain;
  char buf[256];
  int ret;
  char ku, kl, kr, kd;		/* snake parameters */
  /* configuration file format */
  char format[] = "human %c %c %c %c\n";

  rewind (conf);

  while (fgets (buf, 255, conf))
    {
      int playernum;

      /* read configuration file */
      ret = sscanf (buf, format, &kl, &ku, &kr, &kd);

#ifdef DEBUG
      fprintf (stderr, "%s", buf);
      fprintf (stderr, "sscanf returned %i\n", ret);
      fprintf (stderr, "%c, %c, %c, %c\n", kl, ku, kr, kd);
#endif

      /* invalid record format */
      if (ret < 4)
	continue;

      /* make a brain */
      mybrain = make_brain (kl, ku, kr, kd, 0, 0);
      if (!mybrain)
	{			/* out of virtual memory */
	  return 1;
	}

      mysnake = snake_make ();
      mysnake->brain = mybrain;
      mysnake->play = &play;
      playernum = register_player (mysnake);
    }

  return 0;
}

static void
play (snake * s)
{
  brain *mybrain = (brain *) s->brain;

  if (typein == mybrain->kl)
    {
      if (IS_VDIR (s->dir) || s->length == 1)
	s->dir = LEFT;
    }

  else if (typein == mybrain->ku)
    {
      if (IS_HDIR (s->dir) || s->length == 1)
	s->dir = UP;
    }

  else if (typein == mybrain->kr)
    {
      if (IS_VDIR (s->dir) || s->length == 1)
	s->dir = RIGHT;
    }

  else if (typein == mybrain->kd)
    {
      if (IS_HDIR (s->dir) || s->length == 1)
	s->dir = DOWN;
    }
}


/* allocate a brain structure */
static brain *
make_brain (int kl, int ku, int kr, int kd, int ks, int kg)
{
  brain *mybrain;

  mybrain = (brain *) malloc (sizeof (brain));
  if (!mybrain)
    {
      fprintf (stderr, "out of virtual memory\n");
      return NULL;
    }

  mybrain->kl = kl;		/* left key */
  mybrain->ku = ku;		/* up key */
  mybrain->kr = kr;		/* right key */
  mybrain->kd = kd;		/* down key */
  mybrain->ks = ks;		/* stop key */
  mybrain->kg = kg;		/* grow key */

  return mybrain;
}
