#include <stdio.h>
#include <stdlib.h>


/* limits */

#define MAXNAMLEN 50		/* max name length */


/* arena */
#define ARENA_HEIGHT (arenalines())
#define ARENA_WIDTH (arenacols())


/* Directions 
 *
 * Opposite directions MUST BE both odd or even
 */
#define LEFT	0		/* left direction */
#define UP	1		/* up direction */
#define RIGHT	2		/* right direction */
#define DOWN	3		/* down direction */

#define STOP	4		/* no direction (stopped) */

#define IS_FRUIT(x) ( (x) ? ((x)->type == FRUIT) : 0 )
#define IS_SNAKE(x) ( (x) ? ((x)->type == SNAKE) : 0 )
#define IS_EXSNAKE(x) ( (x) ? ((x)->type == EXSNAKE) : 0 )

/* fruit value */
#define FRUITVAL(x) (((x)->value)-48)

/* get sprite at the given coordinate */
#define ARENAGET(y,x) arenaget((y),(x))

#define IS_VDIR(x) (x%2)	/* vertical direction */
#define IS_HDIR(x) (!(x%2))	/* horizontal direction */

#define IS_HORTOGONAL(x,y) ( (x%2) != (y%2) )

/* cell */
typedef struct t_cell
{
  int x;
  int y;
}
cell;

/* snake body (a sprite) */
typedef struct t_snakebody
{
  int type;
  struct t_snake *self;
  struct t_cell cell;
  struct t_snakebody *next;
  struct t_snakebody *prev;
}
snakebody;

/* snake structure */
typedef struct t_snake
{
  int playernum;		/* player number */
  char name[MAXNAMLEN];		/* snake's name */
  int look;			/* snake's look */
  snakebody *head;		/* snake's head chain */
  snakebody *tail;		/* snake's tail chain */
  int length;			/* snake's length */
  int grow;			/* snake's growing state */
  int died;			/* did this snake die? */
  int dir;			/* snake's movement direction */
  unsigned long int score;	/* score */
  void (*play) ();		/* decision taker */
  void *brain;			/* snake's brain */
}
snake;

/* fruit structure */
typedef struct t_fruit
{
  int type;
  struct t_cell cell;
  int value;
}
fruit;

/* MESSAGES ************************************/

/* ARENA ***************************************/
/* sprite types */
#define SNAKE 1
#define EXSNAKE 2
#define FRUIT 3
#define WALL 4

/* sprite template structure */
typedef struct t_sprite
{
  int type;
  char data[1];
}
sprite;


/* external data */
extern FILE *conf;
extern fruit frt;
extern char *rootdir;
extern int typein;

/* external functions */
extern void move_cell (cell *, int, int);
extern int register_player (snake *);
extern snake *snake_make (void);
extern int rand_between (int, int);
extern void message (char *format, ...);

/* arena scanning functions */
extern sprite *arenaget (int y, int x);
extern int arenalines (void);
extern int arenacols (void);
