/*
 *    snake, a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "snake.h"
#include <errno.h>
#include <stdlib.h>		/* malloc */
#include <string.h>		/* strcat, ... */
#include <stdio.h>		/* fopen, fclose, ... */

/* external data */
extern char *rootdir;
extern FILE *conf;

/* private functions */
static char *get_config_path (void);

/* public_functions */
void open_config (void);
void close_config (void);

/* public data */
FILE *conf;

static char *
get_config_path ()
{
  char *confpath;

  confpath = malloc (strlen (rootdir) + strlen (SNAKE_CONFIG) + 5);
  if (!confpath)
    {
      fprintf (stderr, "Out of virtual memory\n");
      finish ();
    }

#ifdef DEBUG
  fprintf (stderr, "snake_config: rootdir before strcat is %s\n", rootdir);
  fprintf (stderr, "snake_config: confpath before strcat is %s\n", confpath);
#endif

  strcpy (confpath, rootdir);
  strcat (confpath, "/");
  strcat (confpath, SNAKE_CONFIG);

#ifdef DEBUG
  fprintf (stderr, "snake_config: rootdir after strcat is %s\n", rootdir);
  fprintf (stderr, "snake_config: confpath after strcat is %s\n", confpath);
#endif

  return confpath;
}

void
open_config ()
{
  char *confpath;

  confpath = get_config_path ();

  conf = fopen (confpath, "r");
  if (!conf)
    {
      fprintf (stderr, "error opening %s: %s\n", confpath, strerror (errno));
      finish ();
    }

}

void
close_config ()
{
  fclose (conf);
}
