/*
 *    snake, a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include "snake.h"
#include <stdlib.h>		/* malloc etc.. */
#include <string.h>		/* string handling funx */

/* external functions */

/* public function */
snake *snake_make (void);

/* allocate space for a new (1-cell-big) snake.
 * initialize snake's structure members
 * assign a random position
 */
snake *
snake_make (void)
{
  int x, y, dir;
  snake *new;			/* new snake pointer */

  do
    {
      x = rand_between (0, ARENA_WIDTH - 1);
      y = rand_between (0, ARENA_HEIGHT - 1);
    }
  while (IS_SNAKE (ARENAGET (y, x)));

  dir = rand_between (0, 3);

  /* allocate memory for the new snake */
  new = (snake *) malloc (sizeof (snake));
  if (!new)
    {				/* no more memory available */
      return NULL;
    }


  /* allocate space for the head/tail */
  new->head = new->tail = (snakebody *) malloc (sizeof (snakebody));

  /* set head/tail types */
  new->head->type = SNAKE;

  /* set self referece */
  new->head->self = new;

  /* set head/tail references to NULL */
  new->head->next = NULL;
  new->head->prev = NULL;

  /* set snake's head and tail chains */
  new->head->cell.y = y;	/* == new->tail->y */
  new->head->cell.x = x;	/* == new->tail->x */

  /* set snake's length */
  new->length = 1;

  /* set fake snake's look (will be set by register_player()) */
  new->look = 'x';

  /* give snake a life */
  new->died = 0;

  /* mark it growing */
  new->grow = 0;

  /* set snake's direction */
  new->dir = dir;

  /* set snake's score */
  new->score = 0;

  /* return the new snake */
  return new;
}
